@@ -1,7 +1,8 @@
 import click
 from opnsense_cli.formatters.cli_output import CliOutputFormatter
 from opnsense_cli.callbacks.click import \
-    formatter_from_formatter_name, bool_as_string, available_formats, int_as_string, tuple_to_csv
+    formatter_from_formatter_name, bool_as_string, available_formats, int_as_string, tuple_to_csv, \
+    resolve_linked_names_to_uuids
 from opnsense_cli.types.click_param_type.int_or_empty import INT_OR_EMPTY
 from opnsense_cli.commands.plugin.haproxy import haproxy
 from opnsense_cli.api.client import ApiClient
@@ -71,12 +72,12 @@ def list(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
         "ssl_cipherList,ssl_cipherSuites,ssl_hstsEnabled,ssl_hstsIncludeSubDomains,ssl_hstsPreload,ssl_hstsMaxAge,"
         "ssl_clientAuthEnabled,ssl_clientAuthVerify,ssl_clientAuthCAs,ssl_clientAuthCRLs,basicAuthEnabled,"
         "Users,basicAuthUsers,Groups,basicAuthGroups,tuning_maxConnections,tuning_timeoutClient,tuning_timeoutHttpReq,"
-        "tuning_timeoutHttpKeepAlive,Cpus,linkedCpuAffinityRules,logging_dontLogNull,logging_dontLogNormal,"
+        "tuning_timeoutHttpKeepAlive,Cpus,linkedCpuAffinityRules,tuning_shards, logging_dontLogNull,logging_dontLogNormal,"
         "logging_logSeparateErrors,logging_detailedLog,logging_socketStats,stickiness_pattern,stickiness_dataTypes,"
         "stickiness_expire,stickiness_size,stickiness_counter,stickiness_counter_key,stickiness_length,"
         "stickiness_connRatePeriod,stickiness_sessRatePeriod,stickiness_httpReqRatePeriod,stickiness_httpErrRatePeriod,"
         "stickiness_bytesInRatePeriod,stickiness_bytesOutRatePeriod,http2Enabled,http2Enabled_nontls,"
-        "advertised_protocols,forwardFor,connectionBehaviour,customOptions,Actions,"
+        "advertised_protocols,forwardFor,prometheus_enabled,prometheus_path,connectionBehaviour,customOptions,Actions,"
         "linkedActions,Errorfiles,linkedErrorfiles"
     ),
     show_default=True,
@@ -138,6 +139,7 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--defaultBackend',
     help=('Set the default backend pool to use for this public service.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None,
     required=False,
@@ -331,6 +333,7 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--basicAuthUsers',
     help=('Set allowed basic auth users.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None,
     required=False,
@@ -338,6 +341,7 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--basicAuthGroups',
     help=('Set allowed basic auth groups.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None,
     required=False,
@@ -387,6 +391,20 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--linkedCpuAffinityRules',
     help=('Choose CPU affinity rules that should be applied to this public service.'),
+    callback=resolve_linked_names_to_uuids,
+    show_default=True,
+    default=None,
+    required=False,
+)
+@click.option(
+    '--tuning_shards',
+    help=(
+            'This option automatically creates the specified number of listeners for every IP:port'
+            ' combination and evenly distributes'
+            ' them among available threads. This can sometimes be useful when using very large thread '
+            ' counts where the in-kernel'
+            ' locking on a single socket starts to cause a significant overhead.'
+    ),
     show_default=True,
     default=None,
     required=False,
@@ -635,6 +653,22 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
     default=True,
     required=True,
 )
+@click.option(
+    '--prometheus_enabled/--no-prometheus_enabled',
+    help=('Enable HAProxy\'s Prometheus exporter.'),
+    show_default=True,
+    is_flag=True,
+    callback=bool_as_string,
+    default=False,
+    required=False,
+)
+@click.option(
+    '--prometheus_path',
+    help=('The path where the Prometheus exporter can be accessed.'),
+    show_default=True,
+    default='/metrics',
+    required=False,
+)
 @click.option(
     '--connectionBehaviour',
     help=(
@@ -647,7 +681,7 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
         'close the outgoing server channel as soon as the server has finished '
         'to respond and release some resources earlier.'
     ),
-    type=click.Choice(['http-keep-alive', 'http-tunnel', 'httpclose', 'http-server-close', 'forceclose']),
+    type=click.Choice(['http-keep-alive', 'httpclose', 'http-server-close']),
     multiple=False,
     callback=tuple_to_csv,
     show_default=True,
@@ -664,6 +698,7 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--linkedActions',
     help=('Choose rules to be included in this public service.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None,
     required=False,
@@ -671,6 +706,7 @@ def show(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--linkedErrorfiles',
     help=('Choose error messages to be included in this public service.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None,
     required=False,
@@ -729,6 +765,7 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
             "tuning_timeoutHttpReq": kwargs['tuning_timeouthttpreq'],
             "tuning_timeoutHttpKeepAlive": kwargs['tuning_timeouthttpkeepalive'],
             "linkedCpuAffinityRules": kwargs['linkedcpuaffinityrules'],
+            "tuning_shards": kwargs['tuning_shards'],
             "logging_dontLogNull": kwargs['logging_dontlognull'],
             "logging_dontLogNormal": kwargs['logging_dontlognormal'],
             "logging_logSeparateErrors": kwargs['logging_logseparateerrors'],
@@ -751,6 +788,8 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
             "http2Enabled_nontls": kwargs['http2enabled_nontls'],
             "advertised_protocols": kwargs['advertised_protocols'],
             "forwardFor": kwargs['forwardfor'],
+            "prometheus_enabled": kwargs['prometheus_enabled'],
+            "prometheus_path": kwargs['prometheus_path'],
             "connectionBehaviour": kwargs['connectionbehaviour'],
             "customOptions": kwargs['customoptions'],
             "linkedActions": kwargs['linkedactions'],
@@ -809,6 +848,7 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--defaultBackend',
     help=('Set the default backend pool to use for this public service.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None
 )
@@ -978,12 +1018,14 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--basicAuthUsers',
     help=('Set allowed basic auth users.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None
 )
 @click.option(
     '--basicAuthGroups',
     help=('Set allowed basic auth groups.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None
 )
@@ -1028,6 +1070,19 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--linkedCpuAffinityRules',
     help=('Choose CPU affinity rules that should be applied to this public service.'),
+    callback=resolve_linked_names_to_uuids,
+    show_default=True,
+    default=None
+)
+@click.option(
+    '--tuning_shards',
+    help=(
+            'This option automatically creates the specified number of listeners for every '
+            ' IP:port combination and evenly distributes'
+            ' them among available threads. This can sometimes be useful when using very large '
+            ' thread counts where the in-kernel'
+            ' locking on a single socket starts to cause a significant overhead.'
+    ),
     show_default=True,
     default=None
 )
@@ -1253,6 +1308,21 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
     callback=bool_as_string,
     default=None
 )
+@click.option(
+    '--prometheus_enabled/--no-prometheus_enabled',
+    help=('Enable HAProxy\'s Prometheus exporter.'),
+    show_default=True,
+    is_flag=True,
+    callback=bool_as_string,
+    default=None
+)
+@click.option(
+    '--prometheus_path',
+    help=('The path where the Prometheus exporter can be accessed.'),
+    show_default=True,
+    default=None,
+    required=False,
+)
 @click.option(
     '--connectionBehaviour',
     help=(
@@ -1265,7 +1335,7 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
         'close the outgoing server channel as soon as the server has finished '
         'to respond and release some resources earlier.'
     ),
-    type=click.Choice(['http-keep-alive', 'http-tunnel', 'httpclose', 'http-server-close', 'forceclose']),
+    type=click.Choice(['http-keep-alive', 'httpclose', 'http-server-close']),
     multiple=False,
     callback=tuple_to_csv,
     show_default=True,
@@ -1280,12 +1350,14 @@ def create(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
 @click.option(
     '--linkedActions',
     help=('Choose rules to be included in this public service.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None
 )
 @click.option(
     '--linkedErrorfiles',
     help=('Choose error messages to be included in this public service.'),
+    callback=resolve_linked_names_to_uuids,
     show_default=True,
     default=None
 )
@@ -1318,13 +1390,14 @@ def update(haproxy_frontend_svc: HaproxyFrontendFacade, **kwargs):
         'ssl_hstsIncludeSubDomains', 'ssl_hstsPreload', 'ssl_hstsMaxAge', 'ssl_clientAuthEnabled',
         'ssl_clientAuthVerify', 'ssl_clientAuthCAs', 'ssl_clientAuthCRLs', 'basicAuthEnabled', 'basicAuthUsers',
         'basicAuthGroups', 'tuning_maxConnections', 'tuning_timeoutClient', 'tuning_timeoutHttpReq',
-        'tuning_timeoutHttpKeepAlive', 'linkedCpuAffinityRules', 'logging_dontLogNull', 'logging_dontLogNormal',
+        'tuning_timeoutHttpKeepAlive', 'linkedCpuAffinityRules', 'tuning_shards', 'logging_dontLogNull',
+        'logging_dontLogNormal',
         'logging_logSeparateErrors', 'logging_detailedLog', 'logging_socketStats', 'stickiness_pattern',
         'stickiness_dataTypes', 'stickiness_expire', 'stickiness_size', 'stickiness_counter', 'stickiness_counter_key',
         'stickiness_length', 'stickiness_connRatePeriod', 'stickiness_sessRatePeriod', 'stickiness_httpReqRatePeriod',
         'stickiness_httpErrRatePeriod', 'stickiness_bytesInRatePeriod', 'stickiness_bytesOutRatePeriod',
-        'http2Enabled', 'http2Enabled_nontls', 'advertised_protocols', 'forwardFor', 'connectionBehaviour',
-        'customOptions', 'linkedActions', 'linkedErrorfiles'
+        'http2Enabled', 'http2Enabled_nontls', 'advertised_protocols', 'forwardFor', 'prometheus_enabled', 'prometheus_path',
+        'connectionBehaviour', 'customOptions', 'linkedActions', 'linkedErrorfiles'
     ]
     for option in options:
         if kwargs[option.lower()] is not None:
